@@ -9,7 +9,7 @@
 #include "globalshortcutcontext.h"
 #include "globalshortcutsregistry.h"
 #include "kglobalaccel_interface.h"
-#include "logging_p.h"
+#include "logging.h"
 #include <config-kglobalaccel.h>
 
 #include <QKeySequence>
@@ -49,10 +49,10 @@ QString Component::stringFromKeys(const QList<QKeySequence> &keys)
     return ret;
 }
 
-Component::Component(const QString &uniqueName, const QString &friendlyName)
+Component::Component(const QString &uniqueName, const QString &friendlyName, GlobalShortcutsRegistry *registry)
     : _uniqueName(uniqueName)
     , _friendlyName(friendlyName)
-    , _registry(GlobalShortcutsRegistry::self())
+    , _registry(registry)
 {
     // Make sure we do no get uniquenames still containing the context
     Q_ASSERT(uniqueName.indexOf(QLatin1Char('|')) == -1);
@@ -171,7 +171,7 @@ void Component::deactivateShortcuts(bool temporarily)
     }
 }
 
-void Component::emitGlobalShortcutPressed(const GlobalShortcut &shortcut)
+void Component::emitGlobalShortcutEvent(const GlobalShortcut &shortcut, ShortcutKeyState state)
 {
 #if HAVE_X11
     // pass X11 timestamp
@@ -184,30 +184,24 @@ void Component::emitGlobalShortcutPressed(const GlobalShortcut &shortcut)
         return;
     }
 
-    Q_EMIT globalShortcutPressed(shortcut.context()->component()->uniqueName(), shortcut.uniqueName(), timestamp);
-}
-
-void Component::emitGlobalShortcutReleased(const GlobalShortcut &shortcut)
-{
-#if HAVE_X11
-    // pass X11 timestamp
-    const long timestamp = QX11Info::appTime();
-#else
-    const long timestamp = 0;
-#endif
-
-    if (shortcut.context()->component() != this) {
-        return;
+    switch (state) {
+    case ShortcutKeyState::Pressed:
+        Q_EMIT globalShortcutPressed(shortcut.context()->component()->uniqueName(), shortcut.uniqueName(), timestamp);
+        break;
+    case ShortcutKeyState::Repeated:
+        Q_EMIT globalShortcutRepeated(shortcut.context()->component()->uniqueName(), shortcut.uniqueName(), timestamp);
+        break;
+    case ShortcutKeyState::Released:
+        Q_EMIT globalShortcutReleased(shortcut.context()->component()->uniqueName(), shortcut.uniqueName(), timestamp);
+        break;
     }
-
-    Q_EMIT globalShortcutReleased(shortcut.context()->component()->uniqueName(), shortcut.uniqueName(), timestamp);
 }
 
 void Component::invokeShortcut(const QString &shortcutName, const QString &context)
 {
     GlobalShortcut *shortcut = getShortcutByName(shortcutName, context);
     if (shortcut) {
-        emitGlobalShortcutPressed(*shortcut);
+        emitGlobalShortcutEvent(*shortcut, ShortcutKeyState::Pressed);
     }
 }
 
@@ -279,7 +273,7 @@ GlobalShortcut *
 Component::registerShortcut(const QString &uniqueName, const QString &friendlyName, const QString &shortcutString, const QString &defaultShortcutString)
 {
     // The shortcut will register itself with us
-    GlobalShortcut *shortcut = new GlobalShortcut(uniqueName, friendlyName, currentContext());
+    GlobalShortcut *shortcut = new GlobalShortcut(uniqueName, friendlyName, currentContext(), _registry);
 
     const QList<QKeySequence> keys = keysFromString(shortcutString);
     shortcut->setDefaultKeys(keysFromString(defaultShortcutString));
@@ -299,7 +293,7 @@ Component::registerShortcut(const QString &uniqueName, const QString &friendlyNa
     return shortcut;
 }
 
-void Component::loadSettings(KConfigGroup &configGroup)
+void Component::loadSettings(const KConfigGroup &configGroup)
 {
     // GlobalShortcutsRegistry::loadSettings handles contexts.
     const auto listKeys = configGroup.keyList();
