@@ -15,7 +15,7 @@
 #include "globalshortcutsregistry.h"
 #include "kglobalaccel.h"
 #include "kserviceactioncomponent.h"
-#include "logging_p.h"
+#include "logging.h"
 
 #include <QDBusConnection>
 #include <QDBusMetaType>
@@ -57,7 +57,7 @@ struct KGlobalAccelDPrivate {
     //! Our holder
     KGlobalAccelD *q;
 
-    GlobalShortcutsRegistry *m_registry = nullptr;
+    std::unique_ptr<GlobalShortcutsRegistry> m_registry = nullptr;
 };
 
 GlobalShortcut *KGlobalAccelDPrivate::findAction(const QStringList &actionId) const
@@ -126,6 +126,11 @@ Component *KGlobalAccelDPrivate::component(const QStringList &actionId) const
     }
 }
 
+KGlobalAccelInterface * ::KGlobalAccelD::interface() const
+{
+    return d->m_registry->interface();
+}
+
 GlobalShortcut *KGlobalAccelDPrivate::addAction(const QStringList &actionId)
 {
     Q_ASSERT(actionId.size() >= 4);
@@ -150,7 +155,10 @@ GlobalShortcut *KGlobalAccelDPrivate::addAction(const QStringList &actionId)
 
     Q_ASSERT(!component->getShortcutByName(componentUnique, contextUnique));
 
-    return new GlobalShortcut(actionId.at(KGlobalAccel::ActionUnique), actionId.at(KGlobalAccel::ActionFriendly), component->shortcutContext(contextUnique));
+    return new GlobalShortcut(actionId.at(KGlobalAccel::ActionUnique),
+                              actionId.at(KGlobalAccel::ActionFriendly),
+                              component->shortcutContext(contextUnique),
+                              m_registry.get());
 }
 
 Q_DECLARE_METATYPE(QStringList)
@@ -172,11 +180,11 @@ bool KGlobalAccelD::init()
     qDBusRegisterMetaType<QList<KGlobalShortcutInfo>>();
     qDBusRegisterMetaType<KGlobalAccel::MatchType>();
 
-    d->m_registry = GlobalShortcutsRegistry::self();
+    d->m_registry = std::make_unique<GlobalShortcutsRegistry>();
     Q_ASSERT(d->m_registry);
 
     d->writeoutTimer.setSingleShot(true);
-    connect(&d->writeoutTimer, &QTimer::timeout, d->m_registry, &GlobalShortcutsRegistry::writeSettings);
+    connect(&d->writeoutTimer, &QTimer::timeout, d->m_registry.get(), &GlobalShortcutsRegistry::writeSettings);
 
     if (!QDBusConnection::sessionBus().registerService(QLatin1String("org.kde.kglobalaccel"))) {
         qCWarning(KGLOBALACCELD) << "Failed to register service org.kde.kglobalaccel";
